#include "dump.h"
#include "bg.h"

//---------------------------------------------------------------------------
ST_DUMP Dump;


//---------------------------------------------------------------------------
IWRAM_CODE void DumpInit(void)
{
	_Memset(&Dump, 0x00, sizeof(ST_DUMP));

	DumpInitSub();
}
//---------------------------------------------------------------------------
IWRAM_CODE void DumpInitSub(void)
{
	u16* pE  = (u16*)DUMP_ROM_END - 1;
	u16* pS  = (u16*)DUMP_ROM_START;
	u16  hex = 0xffff;

	while(pS < pE)
	{
		if(*pE-- != hex--)
		{
			break;
		}
	}
	Dump.size = (((u32)pE + 0xff) & 0xffffff00) - DUMP_ROM_START;

	DumpCrcReset();
	Dump.sizeCRC = DumpCrc(Dump.size);

	Dump.act = DUMP_ACT_LINK;
}
//---------------------------------------------------------------------------
IWRAM_CODE void DumpExec(void)
{
	BgPrintf(0, 0, "ROM Dumper");

	BgPrintf(1, 2, "size    = 0x%08X", Dump.size);
	BgPrintf(1, 3, "sizeCRC = 0x%04X", Dump.sizeCRC);

	BgPrintf(1, 5, "SendCnt = 0x%08X", Dump.SendCnt);
	BgPrintf(1, 6, "dataCRC = 0x%04X", Dump.dataCRC);
}
//---------------------------------------------------------------------------
IWRAM_CODE u32 DumpIntr(void)
{
	switch(Dump.act)
	{
	case DUMP_ACT_LINK:
		Dump.act = DUMP_ACT_SIZE;
		return 0x1234abcd;

	case DUMP_ACT_SIZE:
		Dump.act = DUMP_ACT_SIZE_CRC;
		return Dump.size;

	case DUMP_ACT_SIZE_CRC:
		DumpCrcReset();

		Dump.act = DUMP_ACT_DATA;
		return Dump.sizeCRC;

	case DUMP_ACT_DATA: ;
		u32 ret ALIGN(4);

		ret = *(u32*)(DUMP_ROM_START + Dump.SendCnt);
		Dump.SendCnt += 4;
		Dump.dataCRC  = DumpCrc(ret);

		if(Dump.SendCnt >= Dump.size)
		{
			Dump.act = DUMP_ACT_DATA_CRC;
		}
		return ret;

	case DUMP_ACT_DATA_CRC:
		Dump.act = DUMP_ACT_END;
		return Dump.dataCRC;

	default:
		break;
	}

	return 0xdeadbeef;
}
//---------------------------------------------------------------------------
IWRAM_CODE u32 DumpCrc(u32 d)
{
	u32 bit;

	for(bit=0; bit<32; bit++)
	{
		if((Dump.crc ^ d) & 0x01)
		{
			Dump.crc = (Dump.crc >> 1) ^ 0x0000c37b;
		}
		else
		{
			Dump.crc = (Dump.crc >> 1);
		}

		d = d >> 1;
	}

	return Dump.crc;
}
//---------------------------------------------------------------------------
IWRAM_CODE void DumpCrcReset(void)
{
	Dump.crc = 0x0000c387;
}



/*
BIOS DUMP MEMO

	#define DUMP_SIZE_BIOS 0x4000

	Dump.size = DUMP_SIZE_BIOS;

	u32 i;
	for(i=0; i<Dump.size; i+=4)
	{
		Dump.data[i + 0] = (MidiKey2Freq((void*)(i - 3), 180 - 12, 0) * 2) >> 24;
		Dump.data[i + 1] = (MidiKey2Freq((void*)(i - 2), 180 - 12, 0) * 2) >> 24;
		Dump.data[i + 2] = (MidiKey2Freq((void*)(i - 1), 180 - 12, 0) * 2) >> 24;
		Dump.data[i + 3] = (MidiKey2Freq((void*)(i - 4), 180 - 12, 0) * 2) >> 24;
	}

//---------------------------------------------------------------------------
// call.h

#ifndef __CALL_H__
#define __CALL_H__
#ifdef __cplusplus
extern "C" {
#endif

#include "gba.h"

IWRAM_CODE u32 MidiKey2Freq(void* r0, u8 r1, u8 r2);

#ifdef __cplusplus
}
#endif
#endif


//---------------------------------------------------------------------------
// call.s

	.TEXT
	.CODE 32
	.GLOBAL	MidiKey2Freq
	.FUNC
MidiKey2Freq:
		swi		31 << 16
		bx		lr
	.END
*/
